/*
 * MecanumChassis.cpp
 *
 *  Created on: Oct 5, 2017
 *      Author: jonah
 */

#include <subsys/mecanum/MecanumChassis.h>
#include <subsys/interfaces/IChassis.h>

namespace Team302 {

MecanumChassis::MecanumChassis() :
	IChassis(),
	m_frontLeftMotor(new CANTalon(1)),
	m_frontRightMotor(new CANTalon(0)),
	m_backLeftMotor(new CANTalon(3)),
	m_backRightMotor(new CANTalon(2)),
	m_targetLocationMode(RELATIVE),
	m_movementMode(POWER),
	m_x(0),
	m_y(0),
	m_direction(0),
	m_xTarget(0),
	m_yTarget(0),
	m_directionTarget(0),
	m_pathTravelVelocityTarget(0)
{
	m_frontLeftMotor->SetInverted(true);
	m_backLeftMotor->SetInverted(true);
}

double MecanumChassis::GetVelocity() const {
	return 0;
}

double MecanumChassis::GetXLocation() const {
	return m_x;
}

double MecanumChassis::GetYLocation() const {
	return m_y;
}

double MecanumChassis::GetHeading() const {
	return m_direction;
}

//TODO: remove this because mecanum
IChassis::ChassisType MecanumChassis::GetChassisType() const {
	return MECANUM;
}

void MecanumChassis::SetTargetLocation(double newTargetX, double newTargetY) {
	m_movementMode = LOCATION; //change chassis movement mode to LOCATION
	m_xTarget = newTargetX;
	m_yTarget = newTargetY;
}

void MecanumChassis::SetPathTravelVelocity(double newPathTravelVelocity) {
	m_pathTravelVelocityTarget = newPathTravelVelocity;
}

void MecanumChassis::SetTargetLocationMode(IChassis::TargetLocationMode newTargetLocationMode) {
	m_targetLocationMode = newTargetLocationMode;
}

void MecanumChassis::SetPowerMagnitudes(double xPower, double yPower, double rotationPower) {
	//Since this is the most recent method called, change the movement mode to POWER because
	//we want these powers to control the robot and not the location parameters
	m_movementMode = POWER;
	double frontLeftPower = yPower;
	double backLeftPower = yPower;
	double frontRightPower = yPower;
	double backRightPower = yPower;

	frontLeftPower += rotationPower;
	backLeftPower += rotationPower;
	frontRightPower -= rotationPower;
	backRightPower -= rotationPower;

	frontLeftPower -= xPower;
	backLeftPower += xPower;
	frontRightPower += xPower;
	backRightPower -= xPower;

	double maxValue = 0;
	if (std::abs(frontLeftPower) > maxValue)
		maxValue = std::abs(frontLeftPower);
	if (std::abs(backLeftPower) > maxValue)
			maxValue = std::abs(backLeftPower);
	if (std::abs(frontRightPower) > maxValue)
			maxValue = std::abs(frontRightPower);
	if (std::abs(backRightPower) > maxValue)
				maxValue = std::abs(backRightPower);

	if (maxValue > 1.0) {
		frontLeftPower /= maxValue;
		frontRightPower /= maxValue;
		backLeftPower /= maxValue;
		backRightPower /= maxValue;
	}


	//Apply motor powers to CANTalons
	m_frontLeftMotor->Set(frontLeftPower);
	m_backLeftMotor->Set(backLeftPower);
	m_frontRightMotor->Set(frontRightPower);
	m_backRightMotor->Set(backRightPower);
}

void MecanumChassis::SetLeftRightPower(double leftPower, double rightPower) {
	m_movementMode = POWER; //This is setting power directly to the motors

	m_frontLeftMotor->Set(leftPower);
	m_backLeftMotor->Set(leftPower);
	m_frontRightMotor->Set(rightPower);
	m_backRightMotor->Set(rightPower);
}

void MecanumChassis::UpdateChassis() {
	//update location based on encoder and gyro data

	if (m_movementMode == LOCATION) {
		//do stuff
	}
}

void MecanumChassis::PrintDebug() {
	frc::SmartDashboard::PutNumber("front left vel", m_frontLeftMotor->GetEncVel());
	frc::SmartDashboard::PutNumber("front right vel", m_frontRightMotor->GetEncVel());
	frc::SmartDashboard::PutNumber("back left vel", m_backLeftMotor->GetEncVel());
	frc::SmartDashboard::PutNumber("back right vel", m_backRightMotor->GetEncVel());
}

} /* namespace Team302 */
